import argparse
from pkg_resources import get_distribution

import grpc

import doorctrl_cli.doorctrl_cli_pb2
import doorctrl_cli.doorctrl_cli_pb2_grpc


def cli_parser():
    parser = argparse.ArgumentParser()

    __version__ = get_distribution('doorctrl_cli').version
    parser.add_argument('--version', action='version', version='%(prog)s ' + __version__)

    parser.add_argument('-s', '--server', help="server[:port]")
    parser.add_argument('-c', '--cert')
    parser.add_argument('-k', '--key')
    parser.add_argument('-C', '--ca')

    commands = parser.add_subparsers(dest='command')
    commands.required = True

    commands.add_parser('list', help='list avaiable doors')

    open_parser = commands.add_parser('open', help='open specified door')
    open_parser.add_argument('door')

    req_open_badge_parser = commands.add_parser('request-open-badge',
                                                help='request to open the specified door for the passed key')
    req_open_badge_parser.add_argument('door')
    req_open_badge_parser.add_argument('keyID')

    req_open_user_parser = commands.add_parser('request-open-user',
                                                help='request to open the specified door for the passed username')
    req_open_user_parser.add_argument('door')
    req_open_user_parser.add_argument('username')

    return parser


def error_handler(err):
    code = err.code() if hasattr(err, 'code') else None
    if code == grpc.StatusCode.UNAVAILABLE:
        print("Couldn't establish connection to server.")
    else:
        raise err


def cli():
    parser = cli_parser()
    args = parser.parse_args()

    server = 'unix:/tmp/rfid-kiss.sock'

    if args.server:
        server = args.server

    if args.cert or args.key or args.ca:
        if not args.cert or not args.key or not args.ca:
            parser.error("A secure channel requires -c, -k and -C")

        cred = grpc.ssl_channel_credentials(open(args.ca, 'rb').read(),
                                            open(args.key, 'rb').read(),
                                            open(args.cert, 'rb').read())
        channel = grpc.secure_channel(server, cred)
    else:
        channel = grpc.insecure_channel(server)

    if args.command == 'list':
        stub = doorctrl_cli.doorctrl_cli_pb2_grpc.DoorControllerAdminStub(channel)
        try:
            response = stub.ListDoors(doorctrl_cli.doorctrl_cli_pb2.Empty())
        except grpc.RpcError as err:
            # noinspection PyTypeChecker
            error_handler(err)
            return 1

        print("Available doors:")
        for door in response.doors:
            print('{} {}'.format(door.name, door.accesses))

        return

    elif args.command == 'open':
        stub = doorctrl_cli.doorctrl_cli_pb2_grpc.DoorControllerAdminStub(channel)
        try:
            response = stub.OpenDoor(doorctrl_cli.doorctrl_cli_pb2.OpenDoorRequest(doorName=args.door))
        except grpc.RpcError as err:
            # noinspection PyTypeChecker
            error_handler(err)
            return 1

        print("Status: {}".format(doorctrl_cli.doorctrl_cli_pb2.OpenDoorResponse.Status.Name(response.status)))
        return
    elif args.command == 'request-open-badge':
        stub = doorctrl_cli.doorctrl_cli_pb2_grpc.DoorControllerRemoteStub(channel)
        try:
            response = stub.RequestOpenDoorBadgeID(
                doorctrl_cli.doorctrl_cli_pb2.OpenDoorRequest(doorName=args.door, identifier=args.keyID))
        except grpc.RpcError as err:
            # noinspection PyTypeChecker
            error_handler(err)
            return 1

        print("Status: {}".format(doorctrl_cli.doorctrl_cli_pb2.OpenDoorResponse.Status.Name(response.status)))
        return
    elif args.command == 'request-open-user':
        stub = doorctrl_cli.doorctrl_cli_pb2_grpc.DoorControllerRemoteStub(channel)
        try:
            response = stub.RequestOpenDoorUsername(
                doorctrl_cli.doorctrl_cli_pb2.OpenDoorRequest(doorName=args.door, identifier=args.username))
        except grpc.RpcError as err:
            # noinspection PyTypeChecker
            error_handler(err)
            return 1

        print("Status: {}".format(doorctrl_cli.doorctrl_cli_pb2.OpenDoorResponse.Status.Name(response.status)))
        return
